
import UIKit
import CloudKit

class CitiesViewController: UITableViewController {
   override func viewDidLoad() {
      super.viewDidLoad()
      tableView.register(UITableViewCell.self, forCellReuseIdentifier: "citiesCell")

      Task(priority: .high) {
         await readCities()
      }
      prepareDataSource()
   }
   func readCities() async {
      if AppData.selectedCountry != nil {
         let predicate = NSPredicate(format: "country = %@", AppData.selectedCountry)
         let query = CKQuery(recordType: "Cities", predicate: predicate)

         do {
            let list = try await AppData.database.records(matching: query, inZoneWith: nil, desiredKeys: nil, resultsLimit: 0)
            AppData.listCities = []
            for (_, result) in list.matchResults {
               if let record = try? result.get() {
                  AppData.listCities.append(record)
               }
            }
            await MainActor.run {
               prepareSnapshot()
            }
         } catch {
            print("Error: \(error)")
         }
      }
   }
   func prepareDataSource() {
      AppData.dataSourceCities = UITableViewDiffableDataSource<Sections, CKRecord.ID>(tableView: tableView) { tableView, indexPath, recordID in
         let cell = tableView.dequeueReusableCell(withIdentifier: "citiesCell", for: indexPath)
         if let item = AppData.listCities.first(where: { $0.recordID == recordID }) {
            var config = cell.defaultContentConfiguration()
            config.text = item["name"] as? String
            cell.contentConfiguration = config
         }
         return cell
      }
   }
   func prepareSnapshot() {
      var snapshot = NSDiffableDataSourceSnapshot<Sections, CKRecord.ID>()
      snapshot.appendSections([.main])
      snapshot.appendItems(AppData.listCities.map({ $0.recordID }))
      AppData.dataSourceCities.apply(snapshot)
   }
   @IBAction func addCity(_ sender: UIBarButtonItem) {
      let alert = UIAlertController(title: "Insert City", message: nil, preferredStyle: .alert)
      let cancel = UIAlertAction(title: "Cancel", style: .cancel, handler: nil)
      alert.addAction(cancel)
      let action = UIAlertAction(title: "Save", style: .default, handler: { (action) in
         if let fields = alert.textFields, let name = fields.first?.text {
            Task(priority: .high) {
               await self.insertCity(name: name)
            }
         }
      })
      alert.addAction(action)
      alert.addTextField(configurationHandler: nil)
      present(alert, animated: true, completion: nil)
   }
   func insertCity(name: String) async {
      let text = name.trimmingCharacters(in: .whitespaces)
      if !text.isEmpty {
         let id = CKRecord.ID(recordName: "idcity-\(UUID())")
         let record = CKRecord(recordType: "Cities", recordID: id)
         record.setObject(text as NSString, forKey: "name")
         let reference = CKRecord.Reference(recordID: AppData.selectedCountry, action: .deleteSelf)
         record.setObject(reference, forKey: "country")

         do {
            try await AppData.database.save(record)
            AppData.listCities.append(record)
            
            await MainActor.run {
               prepareSnapshot()
            }
         } catch {
            print("Error: \(error)")
         }
      }
   }
}

