
import UIKit
import CloudKit

class CountriesViewController: UITableViewController {
   override func viewDidLoad() {
      super.viewDidLoad()
      tableView.register(UITableViewCell.self, forCellReuseIdentifier: "countriesCell")

      Task(priority: .high) {
         await readCountries()
      }
      prepareDataSource()
   }
   func readCountries() async {
      let predicate = NSPredicate(format: "TRUEPREDICATE")
      let query = CKQuery(recordType: "Countries", predicate: predicate)
      do {
         let list = try await AppData.database.records(matching: query, inZoneWith: nil, desiredKeys: nil, resultsLimit: 0)
         AppData.listCountries = []
         for (_, result) in list.matchResults {
            if let record = try? result.get() {
               AppData.listCountries.append(record)
            }
         }
         await MainActor.run {
            prepareSnapshot()
         }
      } catch {
         print("Error: \(error)")
      }
   }
   override func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
      performSegue(withIdentifier: "showCities", sender: self)
   }
   override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
      if segue.identifier == "showCities" {
         if let path = self.tableView.indexPathForSelectedRow {
            if let record = AppData.dataSourceCountries.itemIdentifier(for: path) {
               AppData.selectedCountry = record
            }
         }
      }
   }
   func prepareDataSource() {
      AppData.dataSourceCountries = UITableViewDiffableDataSource<Sections, CKRecord.ID>(tableView: tableView) { tableView, indexPath, recordID in
         let cell = tableView.dequeueReusableCell(withIdentifier: "countriesCell", for: indexPath)
         if let item = AppData.listCountries.first(where: { $0.recordID == recordID }) {
            var config = cell.defaultContentConfiguration()
            config.text = item["name"] as? String
            cell.contentConfiguration = config
         }
         return cell
      }
   }
   func prepareSnapshot() {
      var snapshot = NSDiffableDataSourceSnapshot<Sections, CKRecord.ID>()
      snapshot.appendSections([.main])
      snapshot.appendItems(AppData.listCountries.map({ $0.recordID }))
      AppData.dataSourceCountries.apply(snapshot)
   }
   @IBAction func addCountry(_ sender: UIBarButtonItem) {
      let alert = UIAlertController(title: "Insert Country", message: nil, preferredStyle: .alert)
      let cancel = UIAlertAction(title: "Cancel", style: .cancel, handler: nil)
      alert.addAction(cancel)
      let action = UIAlertAction(title: "Save", style: .default, handler: { (action) in
         if let fields = alert.textFields, let name = fields.first?.text {
            Task(priority: .high) {
               await self.insertCountry(name: name)
            }
         }
      })
      alert.addAction(action)
      alert.addTextField(configurationHandler: nil)
      present(alert, animated: true, completion: nil)
   }
   func insertCountry(name: String) async {
      let text = name.trimmingCharacters(in: .whitespaces)
      if !text.isEmpty {
         let id = CKRecord.ID(recordName: "idcountry-\(UUID())")
         let record = CKRecord(recordType: "Countries", recordID: id)
         record.setObject(text as NSString, forKey: "name")
         
         do {
            try await AppData.database.save(record)
            AppData.listCountries.append(record)

            await MainActor.run {
               prepareSnapshot()
            }
         } catch {
            print("Error: \(error)")
         }
      }
   }
}

