
import UIKit
import CoreData

class CountriesViewController: UITableViewController, NSFetchedResultsControllerDelegate {
   var context: NSManagedObjectContext!

   override func viewDidLoad() {
      super.viewDidLoad()
      let app = UIApplication.shared
      let appDelegate = app.delegate as! AppDelegate
      context = appDelegate.persistentContainer.viewContext

      tableView.register(UITableViewCell.self, forCellReuseIdentifier: "countriesCell")
      prepareDataSource()
      prepareFetchedController()
   }
   func prepareDataSource() {
      AppData.dataSourceCountries = UITableViewDiffableDataSource<Sections, NSManagedObjectID>(tableView: tableView) { tableView, indexPath, countryID in
         let cell = tableView.dequeueReusableCell(withIdentifier: "countriesCell", for: indexPath)
         if let country = try? self.context.existingObject(with: countryID) as? Countries {
            var config = cell.defaultContentConfiguration()
            config.text = country.name
            cell.contentConfiguration = config
         }
         return cell
      }
   }
   func prepareFetchedController() {
      let request: NSFetchRequest<Countries> = Countries.fetchRequest()
      let sort = NSSortDescriptor(key: "name", ascending: true, selector: #selector(NSString.caseInsensitiveCompare(_:)))
      request.sortDescriptors = [sort]
      AppData.fetchedControllerCountries = NSFetchedResultsController(fetchRequest: request, managedObjectContext: context, sectionNameKeyPath: nil, cacheName: nil)
      AppData.fetchedControllerCountries.delegate = self
      try? AppData.fetchedControllerCountries.performFetch()
   }
   func controller(_ controller: NSFetchedResultsController<NSFetchRequestResult>, didChangeContentWith snapshot: NSDiffableDataSourceSnapshotReference) {
      let newsnapshot = snapshot as NSDiffableDataSourceSnapshot<Sections, NSManagedObjectID>
      AppData.dataSourceCountries.apply(newsnapshot)
   }
   override func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
      performSegue(withIdentifier: "showCities", sender: self)
   }
   override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
      if segue.identifier == "showCities" {
         if let path = tableView.indexPathForSelectedRow {
            if let countryID = AppData.dataSourceCountries.itemIdentifier(for: path) {
               if let country = try? context.existingObject(with: countryID) as? Countries {
                  AppData.selectedCountry = country
               }
            }
         }
      }
   }
   @IBAction func addCountry(_ sender: UIBarButtonItem) {
      let alert = UIAlertController(title: "Insert Country", message: nil, preferredStyle: .alert)
      let cancel = UIAlertAction(title: "Cancel", style: .cancel, handler: nil)
      alert.addAction(cancel)
      let action = UIAlertAction(title: "Save", style: .default, handler: { (action) in
         if let fields = alert.textFields, let text = fields.first?.text{
            let name = text.trimmingCharacters(in: .whitespaces)
            if !name.isEmpty {
               let newCountry = Countries(context: self.context)
               newCountry.name = name
               try? self.context.save()

               do {
                  try AppData.fetchedControllerCountries.performFetch()
               } catch {
                  print("Error")
               }
            }
         }
      })
      alert.addAction(action)
      alert.addTextField(configurationHandler: nil)
      present(alert, animated: true, completion: nil)
   }
}

