
import UIKit
import CoreData

class CitiesViewController: UITableViewController, NSFetchedResultsControllerDelegate {
   var context: NSManagedObjectContext!

   override func viewDidLoad() {
      super.viewDidLoad()
      let app = UIApplication.shared
      let appDelegate = app.delegate as! AppDelegate
      context = appDelegate.persistentContainer.viewContext
      tableView.register(UITableViewCell.self, forCellReuseIdentifier: "citiesCell")
      prepareDataSource()
      prepareFetchedController()
   }
   func prepareDataSource() {
      AppData.dataSourceCities = UITableViewDiffableDataSource<Sections, NSManagedObjectID>(tableView: tableView) {tableView, indexPath, cityID in
         let cell = tableView.dequeueReusableCell(withIdentifier: "citiesCell", for: indexPath)
         if let city = try? self.context.existingObject(with: cityID) as? Cities {
            var config = cell.defaultContentConfiguration()
            config.text = city.name
            cell.contentConfiguration = config
         }
         return cell
      }
   }
   func prepareFetchedController() {
      let request: NSFetchRequest<Cities> = Cities.fetchRequest()
      request.predicate = NSPredicate(format: "country = %@", AppData.selectedCountry)
      let sort = NSSortDescriptor(key: "name", ascending: true, selector: #selector(NSString.caseInsensitiveCompare(_:)))
      request.sortDescriptors = [sort]
      AppData.fetchedControllerCities = NSFetchedResultsController(fetchRequest: request, managedObjectContext: context, sectionNameKeyPath: nil, cacheName: nil)
      AppData.fetchedControllerCities.delegate = self
      try? AppData.fetchedControllerCities.performFetch()
   }
   func controller(_ controller: NSFetchedResultsController<NSFetchRequestResult>, didChangeContentWith snapshot: NSDiffableDataSourceSnapshotReference) {
      let newsnapshot = snapshot as NSDiffableDataSourceSnapshot<Sections, NSManagedObjectID>
      AppData.dataSourceCities.apply(newsnapshot)
   }
   @IBAction func addCity(_ sender: UIBarButtonItem) {
      let alert = UIAlertController(title: "Insert City", message: nil, preferredStyle: .alert)
      let cancel = UIAlertAction(title: "Cancel", style: .cancel, handler: nil)
      alert.addAction(cancel)
      let action = UIAlertAction(title: "Save", style: .default, handler: { (action) in
         if let fields = alert.textFields, let text = fields.first?.text{
            let name = text.trimmingCharacters(in: .whitespaces)
            if !name.isEmpty {
               let newCity = Cities(context: self.context)
               newCity.name = name
               newCity.country = AppData.selectedCountry
               try? self.context.save()

               do {
                  try AppData.fetchedControllerCities.performFetch()
               } catch {
                  print("Error")
               }
            }
         }
      })
      alert.addAction(action)
      alert.addTextField(configurationHandler: nil)
      present(alert, animated: true, completion: nil)
   }
}

